from dip.model import Bool, Model, observe, Str, Trigger
from dip.ui import (Application, Action, Controller, Form, Menu, MenuBar,
        ToolButton, VBox)
from dip.ui.actions import QuitAction


class ExampleModel(Model):

    # The name.
    name = Str()

    # Triggered when the user wants to quit the application.
    quit = Trigger()

    # The gender.
    male = Bool(True)

    def toggle_quit(self, controller):
        """ Invoked when the action it is bound to is triggered. """

        # Get the toolkit independent object that implements the Quit action.
        action = controller.quit_action

        # Toggle the action's state.
        action.enabled = not action.enabled


class ExampleController(Controller):

    @observe('model.male')
    def __on_male_changed(self, change):
        """ Invoked when the attribute it is bound to is changed. """

        print("Male?:", change.new)

    @observe('model.quit')
    def __on_quit_changed(self, change):
        """ Invoked when the attribute it is bound to is changed. """

        Application().quit()


# Every application needs an Application.
app = Application()

# Create the model.
model = ExampleModel()

# Define the view.
view_factory = VBox(
        MenuBar(
            Menu('quit', title="&File"),
            Menu('toggle_quit',
                    Menu('male', title="Sub-menu"),
                    title="&Examples")),
        Form('name'),
        ToolButton('quit'),
        actions=[QuitAction('quit'), Action('toggle_quit'), Action('male')],
        controller_factory=ExampleController)

# Create an instance of the view bound to the model.
view = view_factory(model)

# Make the instance of the view visible.
view.visible = True

# Enter the event loop.
app.execute()
